@@ -2,7 +2,7 @@
 ## src/common/defs.py
 ##
 ## Copyright (C) 2006 Nikos Kouremenos <kourem AT gmail.com>
-## Copyright (C) 2006-2010 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2006-2012 Yann Leboulanger <asterix AT lagaule.org>
 ## Copyright (C) 2006-2008 Jean-Marie Traissard <jim AT lapin.org>
 ## Copyright (C) 2007 Brendan Taylor <whateley AT gmail.com>
 ##                    Tomasz Melcer <liori AT exroot.org>
@@ -27,7 +27,14 @@ docdir = '../'
 basedir   = '../'
 localedir = '../po'
 
-version = '0.14.4'
+version = '0.15'
+import subprocess
+try:
+    node = subprocess.Popen('hg tip --template "{node|short}"', shell=True,
+        stdout=subprocess.PIPE).communicate()[0]
+    version += '-' + node
+except Exception:
+    pass
 
 import sys, os.path
 for base in ('.', 'common'):
