@@ -1,7 +1,7 @@
 # -*- coding:utf-8 -*-
 ## src/gui_menu_builder.py
 ##
-## Copyright (C) 2009-2010 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2009-2012 Yann Leboulanger <asterix AT lagaule.org>
 ##
 ## This file is part of Gajim.
 ##
@@ -62,7 +62,7 @@ def build_resources_submenu(contacts, account, action, room_jid=None,
 
     return sub_menu
 
-def build_invite_submenu(invite_menuitem, list_):
+def build_invite_submenu(invite_menuitem, list_, ignore_rooms=[]):
     """
     list_ in a list of (contact, account)
     """
@@ -126,6 +126,8 @@ def build_invite_submenu(invite_menuitem, list_):
     message_control.TYPE_GC) + minimized_controls:
         acct = gc_control.account
         room_jid = gc_control.room_jid
+        if room_jid in ignore_rooms:
+            continue
         if room_jid in gajim.gc_connected[acct] and \
         gajim.gc_connected[acct][room_jid] and \
         contacts_transport == gajim.get_transport_name_from_jid(room_jid):
@@ -151,7 +153,7 @@ def build_invite_submenu(invite_menuitem, list_):
 
 def get_contact_menu(contact, account, use_multiple_contacts=True,
 show_start_chat=True, show_encryption=False, show_buttonbar_items=True,
-control=None):
+control=None, gc_contact=None):
     """
     Build contact popup menu for roster and chat window. If control is not set,
     we hide invite_contacts_menuitem
@@ -270,8 +272,9 @@ control=None):
 
     # Unsensitive many items when account is offline
     if gajim.account_is_disconnected(account):
-        for widget in (start_chat_menuitem,     rename_menuitem,
-        edit_groups_menuitem, send_file_menuitem, convert_to_gc_menuitem):
+        for widget in (start_chat_menuitem, rename_menuitem,
+        edit_groups_menuitem, send_file_menuitem, convert_to_gc_menuitem,
+        information_menuitem):
             widget.set_sensitive(False)
 
     if not show_start_chat:
@@ -368,7 +371,14 @@ control=None):
     if muc_icon:
         invite_menuitem.set_image(muc_icon)
 
-    build_invite_submenu(invite_menuitem, [(contact, account)])
+    if gc_contact:
+        if not gc_contact.jid:
+            # it's a pm and we don't know real JID
+            invite_menuitem.set_sensitive(False)
+        else:
+            build_invite_submenu(invite_menuitem, [(gc_contact, account)])
+    else:
+        build_invite_submenu(invite_menuitem, [(contact, account)])
 
     # One or several resource, we do the same for send_custom_status
     status_menuitems = gtk.Menu()
@@ -416,13 +426,17 @@ control=None):
             revoke_auth_menuitem.connect('activate', roster.revoke_auth, jid,
                     account)
 
-    else:
+    elif gajim.connections[account].roster_supported:
         # contact is in group 'Not in Roster'
         add_to_roster_menuitem.set_no_show_all(False)
         subscription_menuitem.set_sensitive(False)
 
         add_to_roster_menuitem.connect('activate', roster.on_add_to_roster,
                 contact, account)
+    else:
+        add_to_roster_menuitem.hide()
+        add_to_roster_menuitem.set_no_show_all(True)
+        subscription_menuitem.set_sensitive(False)
 
     set_custom_avatar_menuitem.connect('activate',
             roster.on_set_custom_avatar_activate, contact, account)
