@@ -1,7 +1,7 @@
 # -*- coding:utf-8 -*-
 ## src/history_window.py
 ##
-## Copyright (C) 2003-2010 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2003-2012 Yann Leboulanger <asterix AT lagaule.org>
 ## Copyright (C) 2005 Vincent Hanquez <tab AT snarc.org>
 ## Copyright (C) 2005-2006 Nikos Kouremenos <kourem AT gmail.com>
 ## Copyright (C) 2006 Dimitur Kirov <dkirov AT gmail.com>
@@ -478,12 +478,17 @@ class HistoryWindow:
 
         if subject:
             message = _('Subject: %s\n') % subject + message
-        message += '\n'
+        xhtml = None
+        if message.startswith('<body '):
+            xhtml = message
+
         if tag_msg:
             self.history_textview.print_real_text(message, [tag_msg],
-                    name=contact_name)
+                    name=contact_name, xhtml=xhtml)
         else:
-            self.history_textview.print_real_text(message, name=contact_name)
+            self.history_textview.print_real_text(message, name=contact_name,
+                xhtml=xhtml)
+        self.history_textview.print_real_text('\n')
 
     def on_query_entry_activate(self, widget):
         text = self.query_entry.get_text()
